# -*- coding: utf-8 -*-
#
# 2023 Tabulon (dev-cdist at tabulon.net)
#
# This file is part of cdist.
#
# cdist is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# cdist is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with cdist. If not, see <http://www.gnu.org/licenses/>.
#
#

import os
import stat

def ensure_file_is_executable_by_all(path):
    """Ensure (and if needed, add) execute permissions
       for everyone (user, group, others) on the given file
       Similar to : chmod a+x <path>
    """
    ensure_file_permissions(path, stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)

def ensure_file_permissions(path, permissions):
    """Ensure (and if needed, add) the given permissions
       for the given filesystem object.
       Similar to using '+' with chmod
    """
    perm = os.stat(path).st_mode & 0o777 # only the last 3 bits relate to permissions

    # If desired permissions were already set, don't meddle.
    if ( (perm & permissions ) != permissions ):
        os.chmod(path, perm | permissions)

    # return a mask of desired permissions that are/were actually set
    return os.stat(path).st_mode & 0o777 & permissions

def file_has_shebang(path):
    """Does the given file start with a shebang ?
    """
    return read_from_file(path, size=2) == '#!'

def read_from_file(path, size=-1, ignore=None):
    """Read and return a number of bytes from the given file.
       If size is '-1' (the default) the entire contents are returned.
    """
    value = ""
    try:
        with open(path, "r") as fd:
            value = fd.read(size)
    except ignore:
        pass
    finally:
        fd.close()
    return value

def slurp_file(path, ignore=None):
    """Read and return the entire contents of a given file
    """
    return read_from_file(path, size=-1, ignore=ignore)
